unit WaveFileWrite;

interface
uses
  Classes,MMSystem,SysUtils,Windows,xmldom, XMLIntf, msxmldom, XMLDoc,Math,
  WaveFile,WaveFileChunk;

type

  TWaveFile_Write = Class
    Private
      WaveXMLRecord:TWaveXMLRecord;
      RIFFchunk_Var:TRIFFchunk;
      BWFchunk_Var:TBWFchunk;
      FMTchunk_Var:TFMTchunk;
      DATAchunk_Var:TDATAchunk;
      DataSize_Var:Int64;
      FileStream:TFileStream;
      FileName_Var:String;
      XML_Var:TWaveXML;   
      Assigned:Boolean;
    Public
      Function AssignFileName(FileName:String):Boolean;
      Property BWFchunk:TBWFchunk Read BWFchunk_Var;
      Property FMTchunk:TFMTchunk Read FMTchunk_Var;
      Procedure AddData(Var Buffer;Size:Int64);
      Procedure WriteData(Var Buffer;Index,Size:Int64);
      Procedure Initialize;
      Property XML:TWaveXML Read XML_Var;
      Procedure CreateXML;
      Property DataSize:Int64 Read DataSize_Var;
      Constructor Create(AOwner:TComponent);Virtual;
      Destructor Destroy;Override;
  end;

implementation

Constructor TWaveFile_Write.Create(AOwner:TComponent);
begin
  Inherited Create;
  FMTchunk_Var:=TFMTchunk.Create;
  BWFchunk_Var:=TBWFchunk.Create;
  DATAchunk_Var:=TDATAchunk.Create(FMTchunk_Var);
  RIFFchunk_Var:=TRIFFchunk.Create(BWFchunk_Var,FMTchunk_Var,DATAchunk_Var);
  DataSize_Var := 0;
  FileName_Var := '';
  Assigned := False;
  XML_Var:=TWaveXML.Create(@WaveXMLRecord);
end;

Destructor TWaveFile_Write.Destroy;
begin
  FMTchunk_Var.Destroy;
  BWFchunk_Var.Destroy;
  DATAchunk_Var.Destroy;
  RIFFchunk_Var.Destroy;
  XML_Var.Destroy;
  Inherited;
end;


Function TWaveFile_Write.AssignFileName(FileName:String):Boolean;
begin
  //if FileExists(FileName) then
  begin
    Result := True;
    Assigned := True;
    Self.FileName_Var := FileName;
  end;
end;

Procedure TWaveFile_Write.AddData(Var Buffer;Size:Int64);
begin
  if FileExists(FileName_Var) then
  begin
    DATAchunk_Var.SetSize(Size+DataSize_Var);
    FileStream:=TFileStream.Create(FileName_Var,fmOpenReadWrite	);
    
    RIFFchunk_Var.Write(FileStream,0,0);
    DATAchunk_Var.Write(FileStream,8+4+BWFchunk_Var.Size+8+FMTchunk_Var.Size+8,0);
    DATAchunk_Var.WriteData(Buffer,FileStream,{RIFFchunk_Var.Size}12+BWFchunk_Var.Size+8+FMTchunk_Var.Size+8+8+DataSize_Var,Size);

    FileStream.Free;
    DataSize_Var := DataSize_Var + Size;
  end;
end;

Procedure TWaveFile_Write.WriteData(Var Buffer;Index,Size:Int64);
begin
  if FileExists(FileName_Var) then
  begin
    if (Index + Size)>DataSize_Var then
    begin
      DATAchunk_Var.SetSize(Index + Size);
    end;
    FileStream:=TFileStream.Create(FileName_Var,fmOpenReadWrite	);
    RIFFchunk_Var.Write(FileStream,0,0);
    DATAchunk_Var.Write(FileStream,8+4+BWFchunk_Var.Size+8+FMTchunk_Var.Size+8,0);
    DATAchunk_Var.WriteData(Buffer,FileStream,{RIFFchunk_Var.Size}12+BWFchunk_Var.Size+8+FMTchunk_Var.Size+8+8+Index,Size);
    FileStream.Destroy;
    DataSize_Var := DataSize_Var + Size;
  end;
end;

Procedure TWaveFile_Write.Initialize;
begin
  BWFchunk_Var.Finalize;
  //if FileExists(FileName_Var) then
  begin
    FileStream:=TFileStream.Create(FileName_Var,fmCreate);
    RIFFchunk_Var.Write(FileStream,0,0);
    FileStream.Destroy;
  end;

  if (FileExists(FileName_Var))and BWFchunk_Var.Active then
  begin
    FileStream:=TFileStream.Create(FileName_Var,fmOpenReadWrite	);
    BWFchunk_Var.Write(FileStream,RIFFchunk_Var.TotalSize,0);
    FileStream.Destroy;
  end;

  if FileExists(FileName_Var) then
  begin
    FileStream:=TFileStream.Create(FileName_Var,fmOpenReadWrite	);
    FMTchunk_Var.Write(FileStream,RIFFchunk_Var.TotalSize+BWFchunk_Var.TotalSize,0);
    FileStream.Destroy;
  end;

  if FileExists(FileName_Var) then
  begin
    FileStream:=TFileStream.Create(FileName_Var,fmOpenReadWrite	);
    DATAchunk_Var.Write(FileStream,RIFFchunk_Var.TotalSize+BWFchunk_Var.TotalSize+FMTchunk_Var.TotalSize,0);
    FileStream.Destroy;
  end;  
end;

Procedure TWaveFile_Write.CreateXML;
var
  FileVar:TextFile;
begin
  if not Assigned then
    Exit;
  assignfile(FileVar,Self.FileName_Var);
  ReWrite(FileVar);
  //ghari_Var:='';
  with Self.WaveXMLRecord do
  begin
    WriteLn(FileVar,'<?xml version="1.0"  encoding="WINDOWS-1256"?><MetaData>');
    WriteLn(FileVar,'<ghari>'+ghari_Var+'</ghari>');
    WriteLn(FileVar,'<NoeMianBarname>'+NoeMianBarname_Var+'</NoeMianBarname>');
    WriteLn(FileVar,'<Mozoo>'+Mozoo_Var+'</Mozoo>');
    WriteLn(FileVar,'<NoeManba>'+NoeManba_Var+'</NoeManba>');
    WriteLn(FileVar,'<onvan>'+onvan_Var+'</onvan>');
    WriteLn(FileVar,'<AddreseArchive>'+AddreseArchive_Var+'</AddreseArchive>');
    WriteLn(FileVar,'<Moddat>'+Moddat_Var+'</Moddat>');
    WriteLn(FileVar,'<GhabeliatePakhsh>'+GhabeliatePakhsh_Var+'</GhabeliatePakhsh>');
    WriteLn(FileVar,'<Tozih>'+Tozih_Var+'</Tozih>');
    WriteLn(FileVar,'<ID>'+ID_Var+'</ID>');
    WriteLn(FileVar,'</MetaData>');
  end;
  closeFile(FileVar);
end;

end.
